#!/bin/bash


JAVA_HOME_JNI=`locate jni.h | head -n1 | sed s/jni.h$//g`
JAVA_HOME_JNIMD=`locate jni_md.h | head -n1 | sed s/jni_md.h$//g`


#########################################
# BUILD THE C++ PART (SAT SOLVER PART)  #
#########################################
cd cppsrc
# unpack picosat (downloaded from http://fmv.jku.at/picosat/)
#tar xvf picosat-961.tar.gz

# build the picosat shared library
if [[ "$OSTYPE" == "darwin"* ]]; then
  cd picosat-961-mac
else 
  cd picosat-961
fi

./configure -shared
make
mv libpicosat.so ../
if [[ "$OSTYPE" == "darwin"* ]]; then
  mv libpicosat.dylib ../
fi
cd ..

# compile the picosat ipasir glue code
gcc -fPIC -Wall -DNDEBUG -O3 -DVERSION=\"961\" -Ipicosat-961 -c ipasirpicosatglue.c

# locate the java directory
#JAVA_HOME_VAR=/Library/Java/JavaVirtualMachines/jdk1.8.0_11.jdk/Contents/Home/include
# compile the JNI code
g++ -o libipasir4j.so -shared -lc -fPIC ipasir4j_Ipasir4jNative.cpp ipasirpicosatglue.o -I$JAVA_HOME_JNI -I$JAVA_HOME_JNIMD -L. -lpicosat

# move the shared libraries
mv libpicosat.so ../
mv libipasir4j.so ../
if [[ "$OSTYPE" == "darwin"* ]]; then
g++ -dynamiclib -o libipasir4j.dylib -shared -lc -fPIC ipasir4j_Ipasir4jNative.cpp ipasirpicosatglue.o -I$JAVA_HOME_JNI -I$JAVA_HOME_JNIMD -L. -lpicosat
  mv libipasir4j.dylib ../
  mv libpicosat.dylib ../
fi
cd ..

#########################################
# BUILD THE Java PART (Java Interface)  #
#########################################

javac -d . src/ipasir4j/*.java
jar -cf ipasir4j.jar ipasir4j/Ipasir4j*
